@@ -1,5 +1,5 @@
 import click
-from pcbnewTransition import pcbnew, isV6
+from pcbnewTransition import pcbnew
 import csv
 import os
 import sys
@@ -21,10 +21,16 @@ def collectBom(components, lscsFields, ignore):
             continue
         if getField(c, "JLCPCB_IGNORE") is not None and getField(c, "JLCPCB_IGNORE") != "":
             continue
+        if hasattr(c, "in_bom") and not c.in_bom:
+            continue
+        if hasattr(c, "on_board") and not c.on_board:
+            continue
+        if hasattr(c, "dnp") and c.dnp:
+            continue
         orderCode = None
         for fieldName in lscsFields:
             orderCode = getField(c, fieldName)
-            if orderCode is not None:
+            if orderCode is not None and orderCode.strip() != "":
                 break
         cType = (
             getField(c, "Value"),
@@ -35,21 +41,29 @@ def collectBom(components, lscsFields, ignore):
     return bom
 
 def bomToCsv(bomData, filename):
-    with open(filename, "w", newline="") as csvfile:
+    with open(filename, "w", newline="", encoding="utf-8") as csvfile:
         writer = csv.writer(csvfile)
         writer.writerow(["Comment", "Designator", "Footprint", "LCSC"])
         for cType, references in bomData.items():
-            value, footprint, lcsc = cType
-            writer.writerow([value, ",".join(references), footprint, lcsc])
+            # JLCPCB allows at most 200 components per line so we have to split
+            # the BOM into multiple lines. Let's make the chunks by 100 just to
+            # be sure.
+            CHUNK_SIZE = 100
+            sortedReferences = sorted(references, key=naturalComponentKey)
+            for i in range(0, len(references), CHUNK_SIZE):
+                refChunk = sortedReferences[i:i+CHUNK_SIZE]
+                value, footprint, lcsc = cType
+                writer.writerow([value, ",".join(refChunk), footprint, lcsc])
 
 def noFilter(footprint):
     return True
 
 def exportJlcpcb(board, outputdir, assembly, schematic, ignore, field,
-           corrections, missingerror, nametemplate, drc):
+           corrections, correctionpatterns, missingerror, nametemplate, drc):
     """
     Prepare fabrication files for JLCPCB including their assembly service
     """
+    ensureValidBoard(board)
     loadedBoard = pcbnew.LoadBoard(board)
 
     if drc:
@@ -62,20 +76,24 @@ def exportJlcpcb(board, outputdir, assembly, schematic, ignore, field,
     gerberdir = os.path.join(outputdir, "gerber")
     shutil.rmtree(gerberdir, ignore_errors=True)
     gerberImpl(board, gerberdir)
-    archiveName = nametemplate.format("gerbers")
+
+    archiveName = expandNameTemplate(nametemplate, "gerbers", loadedBoard)
     shutil.make_archive(os.path.join(outputdir, archiveName), "zip", outputdir, "gerber")
 
     if not assembly:
         return
     if schematic is None:
         raise RuntimeError("When outputing assembly data, schematic is required")
+
+    ensureValidSch(schematic)
+
     correctionFields = [x.strip() for x in corrections.split(",")]
     components = extractComponents(schematic)
     ordercodeFields = [x.strip() for x in field.split(",")]
     bom = collectBom(components, ordercodeFields, refsToIgnore)
 
     posData = collectPosData(loadedBoard, correctionFields,
-        bom=components, posFilter=noFilter)
+        bom=components, posFilter=noFilter, correctionFile=correctionpatterns)
     boardReferences = set([x[0] for x in posData])
     bom = {key: [v for v in val if v in boardReferences] for key, val in bom.items()}
     bom = {key: val for key, val in bom.items() if len(val) > 0}
@@ -91,5 +109,5 @@ def exportJlcpcb(board, outputdir, assembly, schematic, ignore, field,
     if missingFields and missingerror:
         sys.exit("There are components with missing ordercode, aborting")
 
-    posDataToFile(posData, os.path.join(outputdir, nametemplate.format("pos") + ".csv"))
-    bomToCsv(bom, os.path.join(outputdir, nametemplate.format("bom") + ".csv"))
+    posDataToFile(posData, os.path.join(outputdir, expandNameTemplate(nametemplate, "pos", loadedBoard) + ".csv"))
+    bomToCsv(bom, os.path.join(outputdir, expandNameTemplate(nametemplate, "bom", loadedBoard) + ".csv"))
